#!/bin/bash

# Copyright (c) 2016-2019, NVIDIA CORPORATION.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of NVIDIA CORPORATION nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# bringup.sh: framework to validate target board bringup functionality

SCRIPT_DIRECTORY=$(cd `dirname $0` && pwd)
source "${SCRIPT_DIRECTORY}"/bringup_common.sh

# Show the usages text
function show_usage {
	local ScriptName="${1}"

	echo "${SCRIPT_NAME} - Use: ${1} [--interactive|-i] [--help|-h]"\
	" [-m|--module] <MODULE_NAME> -s|--soc <SOC> [-d|--print-dependency]"
cat <<EOF
	This script validates common baseline functionality on a the host board
	Options are:
	--interactive|-i
				   runs in interactive mode, can prompt the user for input
	--help|-h
				   show this help
	--module|-m
				   name of the module to test
	--soc|-s <SOC>
				   name of the SOC
	--print-dependency|-d
				   list pre-req dependencies of tests
EOF
}

function init_var()
{
	SCRIPT_NAME=$(basename "${0}")
	INTERACTIVE_MODE="false"
	PRINT_DEPENDENCY="false"
	SOC=""
	BOARD_SOC=""
	SOC_FAMILY=$(tr -d '\0' < "/proc/device-tree/compatible")
	SELECTED_MODULES=()
	BRINGUP_DIR="$( cd "$( dirname "${0}" )" && pwd )"
	TESTS_DIR="/opt/nvidia/bringup_framework/bringup_tests"
	STAT_FILE="${SCRIPT_DIRECTORY}"/stat.txt
	LOG_FILE="${SCRIPT_DIRECTORY}"/log.txt
	rtc_LOG_FILE="${SCRIPT_DIRECTORY}"/rtc_log.txt
	wdt_LOG_FILE="${SCRIPT_DIRECTORY}"/wdt_log.txt

	# automatically detect SoC if user did not provide it
	if [[ "${SOC_FAMILY}" =~ "tegra210" ]]; then
	   BOARD_SOC="t210"
	elif [[ "${SOC_FAMILY}" =~ "tegra186" ]]; then
	   BOARD_SOC="t186"
	elif [[ "${SOC_FAMILY}" =~ "tegra194" ]]; then
	   BOARD_SOC="t194"
	fi
}

function check_soc {
	if [ ! -z "${SOC}" ]; then
		echo "${SCRIPT_NAME} - Given SoC name is "${SOC}""

		# given SoC did not match system SoC
		if [ "${SOC}" != "${BOARD_SOC}" ]; then
			if [ "${INTERACTIVE_MODE}" == "true" ]; then
				# interactive mode should ask user if want to proceed anyways
				while true; do
					echo -n "${SCRIPT_NAME} - Given SoC does not match system" \
						"SoC (${BOARD_SOC}), continue anyways [Y/N]?"
					read response
					case ${response} in
					[yY])
					break
					;;
					[nN])
					exit 1
					;;
					*)
					;;
					esac
				done
			else
				# automated mode should fail directly
				echo "${SCRIPT_NAME} - [ERROR]: Given SoC does not match" \
					"system SoC (${BOARD_SOC})";
				exit 1
			fi
		fi
	fi
}

function check_kernel_config {
	# get kernel version
	KERNEL_VERSION=$(uname -r | cut -d '.' -f-2)
	CONF_FILE=module_common_"${KERNEL_VERSION}".conf

	# check if conf file exists
	if [ ! -e "${TESTS_DIR}/${CONF_FILE}" ]; then
		echo -e "${SCRIPT_NAME} - [ERROR]: Conf file does not exist on"\
		"system for kernel version ${KERNEL_VERSION}"
		echo "Aborting test..."
		exit 1
	fi
}

# runs tests based on given input
function runModule {
	MODULE_NAME=$(basename "${1}")

	MODULE_BU="\"${1}/bringup_${MODULE_NAME}.sh\""
	MODULE_COMMAND="${MODULE_BU} ${MODULE_NAME} ${INTERACTIVE_MODE} \
		${PRINT_DEPENDENCY} ${SOC}"

	if [ "${PRINT_DEPENDENCY}" == "false" ]; then
		echo ""
		echo "${SCRIPT_NAME} --- --- ---"
		echo "${SCRIPT_NAME} - Running Module: ${MODULE_NAME}"
		echo "${SCRIPT_NAME} - ${MODULE_COMMAND}"
	fi

	# run the module script
	eval "${MODULE_COMMAND}"

	if [ "${PRINT_DEPENDENCY}" == "false" ]; then
		echo "${SCRIPT_NAME} - Completed validation on ${MODULE_NAME} module tests"
	fi
}


# run specific modules && process modules flag input
# automatically detect SoC if user did not provide it
function start_bringup_tests {
	# run the modules
	if [ ${#SELECTED_MODULES[@]} -eq 0 ]; then
		for MODULE_DIR in "${BRINGUP_DIR}"/*/ ;
		do
			MODULE=$(basename "${MODULE_DIR}")
			if [[ "${MODULE}" == "rtc" ]] || [[ "${MODULE}" == "wdt" ]];then
				echo ""
				echo "${SCRIPT_NAME} --- --- ---"
				echo -e "Please run "${MODULE}" module individually using"\
				" "./bringup.sh -m ${MODULE}" "
			else
				runModule "${MODULE_DIR}" 2>&1 | tee -a ${LOG_FILE} ;
			fi
		done
	else
		# user specified modules, then only run those modules
		for MODULE_DIR in "${SELECTED_MODULES[@]}" ;
		do
			MODULE=$(basename "${MODULE_DIR}")
			if [[ "${MODULE}" == "rtc" ]] || [[ "${MODULE}" == "wdt" ]];then
				if [[ "${#SELECTED_MODULES[@]}" != 1 ]];then
					echo ""
					echo "${SCRIPT_NAME} --- --- ---"
					echo -e "Bad option: Please run "${MODULE}" module"\
					"individually using "./bringup.sh -m ${MODULE}" "
				else
					LOG_FILE=${MODULE}_LOG_FILE
					runModule "${BRINGUP_DIR}"/"${MODULE_DIR}"/ 2>&1 | \
					tee -a "${!LOG_FILE}"
				fi
			else
				runModule "${BRINGUP_DIR}"/"${MODULE_DIR}"/ 2>&1 | \
				tee -a ${LOG_FILE} ;
			fi
		done
	fi
}

init_var

# parse the command line arguments
TGETOPT=$(getopt -o ihds:m: --longoptions print-dependency,interactive,help,soc: -n \
	"${SCRIPT_NAME}" -- "${@}")

if [ "${?}" != 0 ]; then
	echo "${SCRIPT_NAME} - Terminating... wrong switch"
	show_usage "${SCRIPT_NAME}"
	exit 1
fi
eval set -- "${TGETOPT}"

# handle the parameters of user input
while [ "${#}" -gt "0" ]; do
	case "${1}" in
		-i|--interactive)
			INTERACTIVE_MODE="true" ;;

		-h|--help)
			show_usage "${SCRIPT_NAME}"
			exit 0 ;;

		-s|--soc)
			shift;
			SOC="${1}"
			;;

		-m|--module)
			# this requires user to put all modules in quotes (""), such as,
			# ./bringup.sh -i -m "gpu cpu" -s t210
			shift;
			allModules=${1};
			for module in ${allModules};
			do
				# make sure all selected module names are valid
				if [ ! -d "${BRINGUP_DIR}/${module}" ]; then
					echo "${SCRIPT_NAME} - [ERROR]: invalid module name: " \
					"${module}"
					exit 1;
				fi

				SELECTED_MODULES+=("${module}")
			done
			;;

		-d|--print-dependency)
			PRINT_DEPENDENCY="true" ;;

		--)
			shift; break ;;

		-*)
			echo "${SCRIPT_NAME} - Terminating... wrong switch: $@" >&2
			show_usage "${SCRIPT_NAME}"
			exit 1 ;;
	esac
	shift
done

# print test info
echo "${SCRIPT_NAME} - Running Bringup Framework Test Suite..."

check_soc
check_kernel_config

# make sure we start with an new and empty stat file
if [ -f "${STAT_FILE}" ]; then
	rm "${STAT_FILE}"
fi

if [ -f "${LOG_FILE}" ]; then
	rm "${LOG_FILE}"
fi

if [ -f "${rtc_LOG_FILE}" ]; then
	rm "${rtc_LOG_FILE}"
fi

if [ -f "${wdt_LOG_FILE}" ]; then
	rm "${wdt_LOG_FILE}"
fi

touch "${STAT_FILE}"
touch "${LOG_FILE}"
touch "${rtc_LOG_FILE}"
touch "${wdt_LOG_FILE}"

start_bringup_tests

if [ "${PRINT_DEPENDENCY}" == "false" ]; then
	print_statistics
fi

echo ""
echo "${SCRIPT_NAME} - Done."
