#!/bin/bash

# Copyright (c) 2016-2018, NVIDIA CORPORATION.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of NVIDIA CORPORATION nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

NUM_PASSED=0
NUM_ALL=0

function run_module_tests
{
	MODULE_NAME="${1}"
	INTERACTIVE="${2}"
	PRINT_DEPENDENCY="${3}"
	SOC="${4}"

	run_module_tests_helper "${MODULE_NAME}" "${INTERACTIVE}" \
		"${PRINT_DEPENDENCY}"

	# run the module script for a specific SOC (if it exists)
	SOC_FILE="${MODULE_NAME}/bringup_${MODULE_NAME}_${SOC}.sh"
	SOC_COMMAND="${SOC_FILE} ${MODULE_NAME} ${INTERACTIVE} \
		${PRINT_DEPENDENCY}"

	# an SOC specific module test may or may not exist
	# only run/call it if is present/exists
	if [ -f "${SOC_FILE}" ]; then
		eval "${SOC_COMMAND}"
	fi
}

function run_module_soc_tests
{
	MODULE_NAME="${1}"
	INTERACTIVE="${2}"
	PRINT_DEPENDENCY="${3}"

	run_module_tests_helper "${MODULE_NAME}" "${INTERACTIVE}" \
		"${PRINT_DEPENDENCY}"
}

function run_module_tests_helper
{
	MODULE_NAME="${1}"
	INTERACTIVE="${2}"
	PRINT_DEPENDENCY="${3}"

	if [ "${PRINT_DEPENDENCY}" == "true" ]; then
		run_print_dependency "${INTERACTIVE}"
	else
		run_tests "${INTERACTIVE}"
		update_module_statistics "total"
		update_module_statistics "${MODULE_NAME}"
	fi
}

function run_tests
{
	INTERACTIVE="${1}"

	Tests_To_Run=()
	if [ "${INTERACTIVE}" == "true" ]; then
		Tests_To_Run=("${INTERACTIVE_TESTS[@]}")
	else
		Tests_To_Run=("${AUTOMATED_TESTS[@]}")
	fi

	for TEST in "${Tests_To_Run[@]}"; do
		echo ""
		echo "${SCRIPT_NAME} - START of test \"${TEST}\""
		eval "${TEST}"
		if [ $? == 0 ]; then
			((NUM_PASSED++));
			echo "${SCRIPT_NAME} - TEST PASSED";
		else
			echo "${SCRIPT_NAME} - TEST FAILED";
		fi
		((NUM_ALL++));
		echo "${SCRIPT_NAME} - END of test \"${TEST}\""
	done
}

function run_print_dependency
{
	INTERACTIVE="${1}"
	if [ "${INTERACTIVE}" == "true" ]; then
		TESTS=("${INTERACTIVE_TESTS[@]}")
	else
		TESTS=("${AUTOMATED_TESTS[@]}")
	fi

	for TEST in "${TESTS[@]}"; do
		echo ""
		eval "${TEST} --print-dependency"
		if [ $? != 0 ]; then
			echo "${SCRIPT_NAME} - ${TEST} dependency unknown";
		fi
	done
}

function update_module_statistics
{
	MODULE=${1}

	# find the file path of the stat output file
	# NOTE: this function should be only called from sub-module scripts
	STAT_FILE="${SCRIPT_DIRECTORY}/../stat.txt"

	MODULE_LINE_NUM=0
	LINE_NUM=0
	SUM_PASSED="${NUM_PASSED}"
	SUM_ALL="${NUM_ALL}"
	while IFS='' read -r LINE || [[ -n "${LINE}" ]]; do
		LINE_NUM=$((LINE_NUM+1))
		LINE_WORDS=(${LINE})
		if [ "${LINE_WORDS[0]}" == "${MODULE}" ]; then
			MODULE_LINE_NUM="${LINE_NUM}"
			SUM_PASSED=$((SUM_PASSED+LINE_WORDS[4]))
			SUM_ALL=$((SUM_ALL+LINE_WORDS[2]))
		fi
	done < "${STAT_FILE}"

	if [ "${MODULE_LINE_NUM}" != 0 ]; then
		sed -i "${MODULE_LINE_NUM}s/.*/${MODULE} Total: ${SUM_ALL} \
		Pass: ${SUM_PASSED}/g" "${STAT_FILE}"
	else
		echo "${MODULE} Total: ${SUM_ALL} Pass: ${SUM_PASSED}" \
		>> "${STAT_FILE}"
	fi
}

function print_statistics {
	STAT_FILE="${SCRIPT_DIRECTORY}/stat.txt"

	echo ""
	echo "==========================Test Results============================="
	while IFS='' read -r LINE || [[ -n "${LINE}" ]]; do
	LINE_NUM=$((LINE_NUM+1))
	LINE_WORDS=(${LINE})
	if [ "${LINE_WORDS[0]}" == "total" ]; then
		echo "Total: ${LINE_WORDS[4]}/${LINE_WORDS[2]} tests PASSED"
		echo ""
		echo "Test breakdown:"
	else
		echo "${LINE_WORDS[0]} ${LINE_WORDS[4]}/${LINE_WORDS[2]} PASSED"
	fi
	done < "${STAT_FILE}"
	echo "==================================================================="
}
