#!/bin/bash

# Copyright (c) 2016-2019, NVIDIA CORPORATION.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of NVIDIA CORPORATION nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

### This file contains a script to package the L4T bringup framework.

# fail on errors
set -e

function init_var {
	SCRIPT_NAME=$(basename "${0}")
	BRINGUP_DIR=""

	DEFAULT_FILES=("bringup.sh"\
		"bringup_common.sh")

	TARBALL_NAME="bringup_framework.tbz2"
	TARBALL_DIRECTORY=""
	FINAL_TARBALL=""

	OUTPUT_DIR_SET=0
	BRINGUP_DIR_SET=0
}

function ShowUsage {
	echo "${SCRIPT_NAME} - Use: "${SCRIPT_NAME}" [--output|-o]" \
		"[--bringup-directory|-b] [--help|-h]"
	cat <<EOF
	This script packages the bringup framework test scripts into a tarball
	Options are:
	--output|-o DIRECTORY
		(Required) Specify which directory the tarball should be saved to.
	--bringup-directory|-b DIRECTORY
		(Required) Specify the location of bringup framework directory.
	--help|-h
		Shows this help
EOF
}

function get_default_files()
{
	pushd "${BRINGUP_DIR}"

	# Check that the default files exist and remove them if they don't
	CHECKED_FILES=()
	for file in "${DEFAULT_FILES[@]}"; do
		if [ ! -e "${file}" ]; then
			echo "${SCRIPT_NAME} - ERROR: "${file}" does not exist and cannot" \
			"be packaged."
			exit 1
		fi
	done

	popd
}

function create_bringup_framework_package()
{
	pushd "${BRINGUP_DIR}"

	# If there is an existing tarball, remove it.
	if [ -e "${FINAL_TARBALL}" ]; then
		rm -f "${FINAL_TARBALL}"
	fi

	# Create the output tarball
	tar -I lbzip2 -vcf "${FINAL_TARBALL}" ../bringup/
	popd
}

init_var
echo "${SCRIPT_NAME} - Running Bringup Framework Packaging"

# Read in the command line arguments
TGETOPT=`getopt -o o:b:h -l output:bringup-directory:help -n \
	"${SCRIPT_NAME}" -- "${@}"`

eval set --"${TGETOPT}"

while [ ${#} -gt 0 ]; do
	case "${1}" in
		-o| --output)
			shift;
			TARBALL_DIRECTORY="${1}"
			if [ ! -d "${TARBALL_DIRECTORY}" ]; then
				echo "${SCRIPT_NAME} - Output directory does not exist."
				exit 1
			fi
			FINAL_TARBALL="${TARBALL_DIRECTORY}/${TARBALL_NAME}"
			OUTPUT_DIR_SET=1
			;;

		-b| --bringup-directory)
			shift;
			BRINGUP_DIR="${1}"
			if [ ! -d "${BRINGUP_DIR}" ]; then
				echo "${SCRIPT_NAME} - Bringup framework directory does" \
					"not exist."
				exit 1
			fi
			BRINGUP_DIR_SET=1
			;;

		-h| --help)
			ShowUsage
			exit 0 ;;

		--)
			shift
			break ;;

		-*)
			echo "${SCRIPT_NAME} - Exiting: Incorrect argument: ${@}"
			ShowUsage
			exit 1 ;;
	esac
	shift
done

# make sure the user has provided both output and bringup directories
if [ "${OUTPUT_DIR_SET}" == 0 ]; then
	echo "${SCRIPT_NAME} - Please specify a valid tarball output directory" \
		"using --output";
	exit 1
fi

if [ "${BRINGUP_DIR_SET}" == 0 ]; then
	echo "${SCRIPT_NAME} - Please specify a valid bringup framework directory" \
		"using --bringup-directory";
	exit 1
fi

get_default_files
create_bringup_framework_package
